from tokenize import String
import pandas as pd
import subprocess
from io import StringIO

def csv_head(path, header):
    if header:
        csv_head = subprocess.check_output(f"sed -n '2p' {path}", shell=True).decode("utf-8").strip()
        return csv_head
    else:
        csv_head = subprocess.check_output(f"head -1 {path}", shell=True).decode("utf-8").strip()
        return csv_head

def csv_tail(path):
    csv_tail = subprocess.check_output(f"tail -1 {path}", shell=True).decode("utf-8").strip()
    return csv_tail

def csv_line(path, n):
    csv_line = subprocess.check_output(f"sed -n '{n}p' {path}", shell=True).decode("utf-8").strip()
    return csv_line

def read_csv_header(path):
    csv_header_str = subprocess.check_output(f"head -1 {path}", shell=True).decode("utf-8").strip()
    return csv_header_str

def read_csv_head(path, header=True):
    head = csv_head(path,header)

    if header:
        csv_header = read_csv_header(path)
        string_data = StringIO(f'{csv_header}\n{head}')
        df = pd.read_csv(string_data, sep=",")
        return df
    else:
        string_data = StringIO(f'{head}')
        df = pd.read_csv(string_data, sep=",", header=None)
        return df


def read_csv_tail(path, header=True):
    tail = csv_tail(path)

    if header:
        csv_header = read_csv_header(path)
        string_data = StringIO(f'{csv_header}\n{tail}')
        df = pd.read_csv(string_data, sep=",")
        return df
    else:
        string_data = StringIO(f'{tail}')
        df = pd.read_csv(string_data, sep=",", header=None)
        return df

def read_csv_headtail(path, header=True):
    head = csv_head(path,header)
    tail = csv_tail(path)

    if header:
        csv_header = read_csv_header(path)
        string_data = StringIO(f'{csv_header}\n{head}\n{tail}')
        df = pd.read_csv(string_data, sep=",")
        return df
    else:
        string_data = StringIO(f'{head}\n{tail}')
        df = pd.read_csv(string_data, sep=",", header=None)
        return df

def read_csv_line(path, n, header=True):
    line = csv_line(path, n)
    if header:
        csv_header = read_csv_header(path)
        string_data = StringIO(f'{csv_header}\n{line}')
        df = pd.read_csv(string_data, sep=",")
        return df
    else:
        string_data = StringIO(f'{line}')
        df = pd.read_csv(string_data, sep=",", header=None)
        return df



import pandas as pd
import time
from pathlib import Path
import readcsvturbo as rct

################### Reading the first and last line from big_csv and getting a df ###################
csv_file_path = Path("./big_csv.csv")

print("""
############## RAW PANDAS ###############
# Read the whole file in using read_csv #
#########################################
""")

start_time = time.perf_counter()

df = pd.read_csv(csv_file_path)

df_first_line = df.iloc[[0]]
df_last_line = df.iloc[[-1]]

frames = [df_first_line, df_last_line]
df = pd.concat(frames)

print(df)

end_time = time.perf_counter()

print(f'RAW PANDAS TIME: {round(end_time-start_time,2)}s\n')


print("""
################### SKIPROWS ##################
# Get the length of the file from readlines() #
# Read big_csv using skiprows                 #
###############################################
""")


start_time = time.perf_counter()

count=len(open(csv_file_path).readlines())
df = pd.read_csv(csv_file_path, skiprows=range(2,count-1), header=0)

print(df)

end_time = time.perf_counter()

print(f'SKIPROWS TIME: {round(end_time-start_time,2)}s\n')

print("""
##################### TURBO #####################
# Read first and last line using read-csv-turbo #
#################################################
""")


start_time = time.perf_counter()

df = rct.read_csv_headtail(csv_file_path)
print(df)

end_time = time.perf_counter()

print(f'TURBO TIME: {round(end_time-start_time,2)}s\n')


import pandas as pd
from pathlib import Path
from readcsvturbo import read_csv_head, read_csv_headtail, read_csv_line, read_csv_tail

path = Path("/home/donjor/5mSalesRecords.csv")

df = read_csv_head(path)
print(df)
df = read_csv_headtail(path)
print(df)
df = read_csv_line(path,100)
print(df)
df = read_csv_tail(path)
print(df)

df = read_csv_head(path,False)
print(df)
df = read_csv_headtail(path,False)
print(df)
df = read_csv_line(path,100,False)
print(df)
df = read_csv_tail(path,False)
print(df)

df = read_csv_headtail(path)

number = df['Total Profit'].iloc[0]
number2 = df['Total Profit'].iloc[1]

sum = number + number2
print(sum)


from setuptools import setup, find_packages

VERSION = '0.0.5' 
DESCRIPTION = 'Improved method for reading the first/last/specific line from csv into a DataFrame'
LONG_DESCRIPTION = """Improved method for reading the first/last/specific line from csv into a DataFrame

Ever deal with multiple huge csv files and and the panads `read_csv/skiprows` method is slowing you down? You are not alone.

read-csv-turbo is an improved method of reading the first and last lines using unix `head` and `tail` commands to get the data you want in a dataframe as fast as possible. I may include Windows support in the future if requested.

Reading a large csv once is "fine" but often I find myself looping through many files and this process is painfully slow which is why StackOverflow suggestions didn't cut it. There may be a newer/smarter way of approaching this but this method should be as fast as you could get. 

At the moment the use case of this is quite limited as it just provides a fast way to read the `first`, `last` or `n` row of a csv into a dataframe

github: https://github.com/donjor/read-csv-turbo
"""

# Setting up
setup(
        name="readcsvturbo", 
        version=VERSION,
        author="Donjor",
        author_email="donjordev@gmail.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=["pandas"],
        url="https://github.com/donjor/read-csv-turbo",

        keywords=['python', 'pandas', 'readcsv', 'readfirstlinecsv', 'readlastlinecsv', 'readspecificlinecsv'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: POSIX :: Linux",
        ]
)